package diffusioncalculator;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.panel.CrosshairOverlay;
import org.jfree.chart.plot.Crosshair;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleEdge;

/* Class making all GUI concerning the Permeation chart. 
 * Setting up layouts, applying crosshairs and making components.
 * Functions used to create the chart, buttons, and registering 
 * mouse activity*/
@SuppressWarnings("serial")
public class PermeationChartView extends JFrame implements ChartMouseListener {

	// Declaring the JFREECHART chart and panel
	JFreeChart chart;
	ChartPanel chartPanel;

	// Declaring buttons and panel to attach button onto
	JButton lowPassButton;
	JButton highPassButton;
	JButton restoreButton;
	JPanel buttonPanel;

	// Declaring the two crosshairs and an overlay applied to the chart
	Crosshair xCrosshair;
	Crosshair yCrosshair;
	CrosshairOverlay crosshairOverlay;

	// Declaring a PermeationChartModel instance, containing all data
	PermeationChartModel permeationModel;

	// Constructor creating the frame GUI
	public PermeationChartView(final String title,
			PermeationChartModel inputPermeationModel) {

		// declaring an instance of the CurrentCharModel from input
		permeationModel = inputPermeationModel;

		// creating the chart and the chart panel
		chart = createChart(permeationModel.dataset);
		chartPanel = new ChartPanel(chart);
		chartPanel.setLayout(null);

		// creating the button panel
		buttonPanel = new JPanel();
		buttonPanel.setLayout(null);

		// creating and overlayong crosshairs to the chart panel
		this.chartPanel.addChartMouseListener(this);
		crosshairOverlay = new CrosshairOverlay();
		this.xCrosshair = new Crosshair(Double.NaN, Color.GRAY,
				new BasicStroke(0f));
		this.xCrosshair.setLabelVisible(true);
		this.yCrosshair = new Crosshair(Double.NaN, Color.GRAY,
				new BasicStroke(0f));
		this.yCrosshair.setLabelVisible(true);
		crosshairOverlay.addDomainCrosshair(xCrosshair);
		crosshairOverlay.addRangeCrosshair(yCrosshair);
		chartPanel.addOverlay(crosshairOverlay);

		// Setting boundaries and adding the chart panel to the frame
		chartPanel.setBounds(0, 0, (int) (Util.XSIZE * 0.95),
				(int) (Util.YSIZE * 0.8));
		add(chartPanel);

		// Creating the buttons, setting boundaries, and adding them to the
		// buttonPanel
		lowPassButton();
		lowPassButton.setBounds(0, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(lowPassButton);
		highPassButton();
		highPassButton.setBounds(230, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(highPassButton);
		restoreButton();
		restoreButton.setBounds(460, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(restoreButton);

		// Setting boundaries for the buttonPanel and adding it to the frame
		buttonPanel.setBounds(0, (int) (Util.YSIZE * 0.8),
				(int) (Util.XSIZE * 0.95), (int) (Util.YSIZE * 0.2));
		add(buttonPanel);

		// Setting the size of the frame and making it visible
		setSize(Util.XSIZE, (int) (Util.YSIZE * 0.9));
		setVisible(true);

	}

	// Creating the chart
	private JFreeChart createChart(final XYDataset dataset) {

		// Initializing the chart
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"Permeation Plot", // chart title
				"tDl/L^2", // x axis label
				"J(t)/Jss", // y axis label
				dataset, // data
				PlotOrientation.VERTICAL, true, // include legend
				true, // tooltips
				false // urls
				);

		// Customization of the chart
		chart.setBackgroundPaint(Color.white);

		// Make a reference to the plot for further customization
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.lightGray);

		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);

		LogarithmicAxis xAxis = new LogarithmicAxis("tDl/L^2");
		plot.setDomainAxis(xAxis);

		// Rendering the line representation
		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, true);
		renderer.setSeriesShapesVisible(0, false);
		plot.setRenderer(renderer);

		return chart;

	}

	@Override
	public void chartMouseClicked(ChartMouseEvent event) {
		// TODO Auto-generated method stub

	}

	@Override
	public void chartMouseMoved(ChartMouseEvent event) {
		Rectangle2D dataArea = this.chartPanel.getScreenDataArea();
		JFreeChart chart = event.getChart();
		XYPlot plot = (XYPlot) chart.getPlot();
		ValueAxis xAxis = plot.getDomainAxis();
		double x = xAxis.java2DToValue(event.getTrigger().getX(), dataArea,
				RectangleEdge.BOTTOM);
		double y = DatasetUtilities.findYValue(plot.getDataset(), 0, x);
		this.xCrosshair.setValue(x);
		this.yCrosshair.setValue(y);

	}

	// Create button for low pass filter
	public void lowPassButton() {
		lowPassButton = new JButton("Low Pass Filter");

		// When low pass button is pressed
		lowPassButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				permeationModel.lowPassFilter();
			}
		});
	}

	// Create button for low pass filter
	public void highPassButton() {
		highPassButton = new JButton("High Pass Filter");

		// When low pass button is pressed
		highPassButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				permeationModel.highPassFilter();
			}
		});
	}

	// Create button to restore original
	public void restoreButton() {
		restoreButton = new JButton("Restore");

		// When restore button is pressed
		restoreButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				permeationModel.restore();
			}
		});
	}

}