package diffusioncalculator;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import javax.swing.JTextArea;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART framework to make the data set plot in the Current/Time chart. 
 * Implements Fick's lattice curve, elapsed time curve, and numerical fitting, and 
 * Lino's numerical fitting, with related low pass and high pass filter.
 * Also implements the custom low pass filter, and the custom high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class CurrentChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> time;
	public ArrayList<Float> current;

	// Declaring a list used to hold results from Linos method
	// and a button to check if Linos have been runned
	ArrayList<Double> endList;
	Boolean linoRun = false;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries transient1;
	XYSeries transient2;
	XYSeries transient3;
	XYSeries ficksCurve;
	XYSeries ficksElapsed;
	XYSeries ficksFit;
	XYSeries linosFit;

	// Declaring recurring variables
	int index;
	float deff;
	float L;
	float dl;
	float ficksDeff;
	float linosDeff;
	double linosKappa;
	double linosLambda;
	double linosMy;
	double linosResidual;

	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public CurrentChartModel(ArrayList<Float> inputTime,
			ArrayList<Float> inputCurrent, float inputL, float inputDeff,
			float inputDl, int inputIndex) {

		// Initializing the variables and lists from input
		index = inputIndex;
		deff = inputDeff;
		L = inputL;
		dl = inputDl;
		time = inputTime;
		current = inputCurrent;

		// Initializing the JFreeChart series
		transient1 = new XYSeries("Transient 1");
		transient2 = new XYSeries("Transient 2");
		transient3 = new XYSeries("Transient 3");
		ficksCurve = new XYSeries("Fick's Lattice Curve");
		ficksElapsed = new XYSeries("Fick's Elapsed Time Curve");
		ficksFit = new XYSeries("Fick's Curve Fitting");
		linosFit = new XYSeries("Lino's Curve Fitting");

		// Adding x and y values to the first transient using the restore
		// function
		restore();

		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(transient1);
	}

	// Method to make Fick's lattice curve, based on Fourier series
	public void ficksCurve() {

		// Declaring and initializing the scaled max current
		double maxCurrent = (double) Collections.max(current) * 1000000;

		// Remove potential existing data
		ficksCurve.clear();
		dataset.removeSeries(ficksCurve);

		// Declare local variables
		double temp;
		double ficks;

		// Computing and adding value for each experimental element
		for (double i = 1; i < time.size(); i++) {
			ficks = (1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((3 * i * dl) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((3 * i * dl) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((3 * i * dl) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((3 * i * dl) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((3 * i * dl) / (L * L))) - Math.exp(-Math.PI
					* Math.PI * ((3 * i * dl) / (L * L)))))
					* maxCurrent;
			temp = (time.get((int) i));
			ficksCurve.add(temp, ficks);
		}

		// Add series to dataset
		dataset.addSeries(ficksCurve);
	}

	// Method to make Fick's Elapsed time curve, based on Fourier series
	public void ficksElapsed() {

		// Declaring and initializing the scaled max current
		double maxCurrent = (double) Collections.max(current) * 1000000;

		// Remove potential existing data
		ficksElapsed.clear();
		dataset.removeSeries(ficksElapsed);

		// Declare local variables
		double temp;
		double ficks;

		// Computing and adding value for each experimental element
		for (double i = 1; i < time.size(); i++) {
			ficks = (1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((3 * i * deff) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L))) - Math.exp(-Math.PI
					* Math.PI * ((3 * i * deff) / (L * L)))))
					* maxCurrent;
			temp = (time.get((int) i));
			ficksElapsed.add(temp, ficks);
		}

		// Add series to dataset
		dataset.addSeries(ficksElapsed);
	}

	// Partial computation from linos model, utilized by linos methods
	public double linosMethod(double[] snneg, double[] snpos, double lambda,
			double my, double i, double deff) {
		double linos2 = -((2 * Math.pow(1, 2) * Math.PI * Math.PI * Math
				.exp(-snneg[0] * ((3 * i * deff) / (L * L)))) / (snneg[0] * (1 + ((lambda * my) / Math
				.pow(snneg[0] - my, 2)))))
				+ ((2 * Math.pow(2, 2) * Math.PI * Math.PI * Math.exp(-snneg[1]
						* ((3 * i * deff) / (L * L)))) / (snneg[1] * (1 + ((lambda * my) / Math
						.pow(snneg[1] - my, 2)))))
				- ((2 * Math.pow(3, 2) * Math.PI * Math.PI * Math.exp(-snneg[2]
						* ((3 * i * deff) / (L * L)))) / (snneg[2] * (1 + ((lambda * my) / Math
						.pow(snneg[2] - my, 2)))))
				+ ((2 * Math.pow(4, 2) * Math.PI * Math.PI * Math.exp(-snneg[3]
						* ((3 * i * deff) / (L * L)))) / (snneg[3] * (1 + ((lambda * my) / Math
						.pow(snneg[3] - my, 2)))))
				- ((2 * Math.pow(5, 2) * Math.PI * Math.PI * Math.exp(-snneg[4]
						* ((3 * i * deff) / (L * L)))) / (snneg[4] * (1 + ((lambda * my) / Math
						.pow(snneg[4] - my, 2)))))
				+ ((2 * Math.pow(6, 2) * Math.PI * Math.PI * Math.exp(-snneg[5]
						* ((3 * i * deff) / (L * L)))) / (snneg[5] * (1 + ((lambda * my) / Math
						.pow(snneg[5] - my, 2)))));

		double linos3 = -((2 * Math.pow(1, 2) * Math.PI * Math.PI * Math
				.exp(-snpos[0] * ((3 * i * deff) / (L * L)))) / (snpos[0] * (1 + ((lambda * my) / Math
				.pow(snpos[0] - my, 2)))))
				+ ((2 * Math.pow(2, 2) * Math.PI * Math.PI * Math.exp(-snpos[1]
						* ((3 * i * deff) / (L * L)))) / (snpos[1] * (1 + ((lambda * my) / Math
						.pow(snpos[1] - my, 2)))))
				- ((2 * Math.pow(3, 2) * Math.PI * Math.PI * Math.exp(-snpos[2]
						* ((3 * i * deff) / (L * L)))) / (snpos[2] * (1 + ((lambda * my) / Math
						.pow(snpos[2] - my, 2)))))
				+ ((2 * Math.pow(4, 2) * Math.PI * Math.PI * Math.exp(-snpos[3]
						* ((3 * i * deff) / (L * L)))) / (snpos[3] * (1 + ((lambda * my) / Math
						.pow(snpos[3] - my, 2)))))
				- ((2 * Math.pow(5, 2) * Math.PI * Math.PI * Math.exp(-snpos[4]
						* ((3 * i * deff) / (L * L)))) / (snpos[4] * (1 + ((lambda * my) / Math
						.pow(snpos[4] - my, 2)))))
				+ ((2 * Math.pow(6, 2) * Math.PI * Math.PI * Math.exp(-snpos[5]
						* ((3 * i * deff) / (L * L)))) / (snpos[5] * (1 + ((lambda * my) / Math
						.pow(snpos[5] - my, 2)))));
		return linos2 + linos3;
	}

	// Setting boundary and initial conditions, and running the Linos model
	// three times
	// with different conditions
	public void linosConditions() {
		// Initializing array holding all conditions
		double[] conditions = new double[] { 5, 1, deff, deff / 10, deff * 10,
				1E-7, 1, 10000, 100, 0, 2000, 10 };
		// First run through of model
		conditions = linosFit(conditions);
		System.out.println(Arrays.toString(conditions));

		// Changing conditions
		conditions[3] = conditions[2] - 1E-7;
		conditions[4] = conditions[2] + 1E-7;
		conditions[5] = 1E-8;
		conditions[6] = conditions[0] - 200;
		conditions[7] = conditions[0] + 200;
		conditions[8] = 10;
		conditions[9] = conditions[1] - 50;
		conditions[10] = conditions[1] + 50;
		conditions[11] = 1;

		// Second run through
		conditions = linosFit(conditions);
		System.out.println(Arrays.toString(conditions));

		// Changing conditions
		conditions[3] = conditions[2] - 1E-8;
		conditions[4] = conditions[2] + 1E-8;
		conditions[5] = 1E-9;
		conditions[6] = conditions[0] - 10;
		conditions[7] = conditions[0] + 10;
		conditions[8] = 1;
		conditions[9] = conditions[1] - 5;
		conditions[10] = conditions[1] + 5;
		conditions[11] = 0.1;

		// last run through
		conditions = linosFit(conditions);
		System.out.println(Arrays.toString(conditions));

		// Setting flag value
		linoRun = true;
	}

	public double[] linosFit(double[] conditions) {

		// Declaring and initializing the scaled max current
		double maxCurrent = (double) Collections.max(current) * 1000000;

		// Declaring max current value from experiment
		int imax = current.indexOf(Collections.max(current));

		// Initializing result holding list
		endList = new ArrayList<Double>();

		// Remove potential existing data
		linosFit.clear();
		dataset.removeSeries(linosFit);

		// Declare and initialize variables
		double linos;
		double kappa = 0.0001;
		double lambda = conditions[0];
		double my = conditions[1];
		double newDeff = 0;
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double sn1;
		double sn2;
		double temp;
		double newMy = 0;
		double newLambda = 0;
		double R = 50;

		// Use method to define effective diffusion coefficient by curve fitting
		newDeff = linosDeff(kappa, lambda, my, imax, conditions);

		// Checking and changing boundary conditions if below 0
		if (conditions[6] < 0) {
			conditions[6] = 1;
		}
		if (conditions[9] < 0) {
			conditions[9] = 0;
		}

		// Incrementing lambda value, and the my value. Adding results to list
		for (lambda = conditions[6]; lambda < conditions[7]; lambda += conditions[8]) {
			System.out.println(lambda);
			newMy = linosMy(kappa, lambda, my, newDeff, imax, conditions);
			newLambda = lambda;
			endList.add(newDeff);
			endList.add(newMy);
			endList.add(newLambda);
		}

		// Printing result list, and identifying best fit
		for (int k = 0; k < endList.size(); k += 4) {
			System.out.println("nr: " + k / 3 + " R: " + endList.get(k)
					+ " deff: " + endList.get(k + 1) + " my: "
					+ endList.get(k + 2) + " lambda: " + endList.get(k + 3)
					+ " kappa: " + kappa);
			if (endList.get(k) < R) {
				R = endList.get(k);
				newDeff = endList.get(k + 1);
				newMy = endList.get(k + 2);
				newLambda = endList.get(k + 3);
			}
		}

		// Computing first part of the linos model, and helping functions
		double linos1 = linos1(kappa);
		for (int n = 1; n < 7; n++) {
			sn1 = n * n * Math.PI * Math.PI + kappa + newLambda + newMy;
			sn2 = Math.sqrt(Math.pow(n * n * Math.PI * Math.PI + kappa
					+ newLambda - newMy, 2)
					+ 4 * newLambda * newMy);
			snpos[n - 1] = 0.5 * (sn1 + sn2);
			snneg[n - 1] = 0.5 * (sn1 - sn2);
		}

		// Printing parameter results
		System.out.println("linos1: " + linos1 + " R: " + R + " newDeff: "
				+ newDeff + " newMy: " + newMy + " newLambda: " + newLambda
				+ " newKappa: " + kappa);

		// Saving results to list in order to improve conditions
		conditions[0] = newLambda;
		conditions[1] = newMy;
		conditions[2] = newDeff;

		// Saving values to ResultView presentables
		linosDeff = (float) newDeff;
		linosMy = newMy;
		linosLambda = newLambda;
		linosKappa = kappa;
		linosResidual = R;

		// Computing and adding value for each experimental element
		for (int i = 0; i < time.size(); i++) {
			linos = (linos1 + linosMethod(snneg, snpos, newLambda, newMy, i,
					newDeff)) * maxCurrent;
			temp = (time.get((int) i));
			linosFit.add(temp, linos);
		}

		// Add series to dataset
		dataset.addSeries(linosFit);

		// Clear the result hodling list
		endList.clear();

		// Returning boundary and initial conditions
		return conditions;
	}

	// Function computing first part of the linos model
	public double linos1(double kappa) {
		return (Math.sqrt(kappa)) / (Math.sinh(Math.sqrt(kappa)));
	}

	public double linosDeff(double kappa, double lambda, double my, int imax,
			double[] conditions) {

		// Declaring and initializing the scaled max current
		double maxCurrent = (double) Collections.max(current);

		// Initializing lists to hold residuals and diffusion coefficients
		ArrayList<Double> rList = new ArrayList<Double>();
		ArrayList<Double> list = new ArrayList<Double>();

		// Declare and initialize variables
		double sn1;
		double sn2;
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double linos;
		double R = 0;

		// Computing first part of the linos model, and helping functions
		double linos1 = linos1(kappa);
		for (int n = 1; n < 7; n++) {
			sn1 = n * n * Math.PI * Math.PI + kappa + lambda + my;
			sn2 = Math.sqrt(Math.pow(n * n * Math.PI * Math.PI + kappa + lambda
					- my, 2)
					+ 4 * lambda * my);
			snpos[n - 1] = 0.5 * (sn1 + sn2);
			snneg[n - 1] = 0.5 * (sn1 - sn2);
		}

		// Finding the best match for the diffusion coefficient, using boundary
		// and initial conditions,
		// and the least Squares method. The diffusion coefficient and residual
		// are saved to lists
		for (double newDeff = conditions[3]; newDeff < conditions[4]; newDeff += conditions[5]) {
			R = 0;
			for (int i = (int) (0.005 * time.size()); i < imax; i++) {
				linos = (linos1 + linosMethod(snneg, snpos, lambda, my, i,
						newDeff)) * maxCurrent;
				R += Math.pow((current.get((int) i) - linos), 2);
			}
			rList.add((double) R);
			list.add(newDeff);
		}

		// Retrieving index of best match, and returning it
		int minIndex = rList.indexOf(Collections.min(rList));
		return list.get(minIndex);
	}

	public double linosMy(double kappa, double lambda, double my,
			double newDeff, int imax, double[] conditions) {

		// Declaring and initializing the scaled max current
		double maxCurrent = (double) Collections.max(current);

		// Initializing lists to hold residuals and diffusion coefficients
		ArrayList<Double> rList = new ArrayList<Double>();
		ArrayList<Double> list = new ArrayList<Double>();

		// Declare and initialize variables
		double sn1;
		double sn2;
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double linos;
		float R = 0;

		// Computing first part of the linos model
		double linos1 = linos1(kappa);

		// Finding the best match for the my value, using boundary and initial
		// conditions,
		// and the least Squares method. The my value and residual are saved to
		// lists
		for (my = conditions[9]; my < conditions[10]; my += conditions[11]) {
			for (int n = 1; n < 7; n++) {
				sn1 = n * n * Math.PI * Math.PI + kappa + lambda + my;
				sn2 = Math.sqrt(Math.pow(n * n * Math.PI * Math.PI + kappa
						+ lambda - my, 2)
						+ 4 * lambda * my);
				snpos[n - 1] = 0.5 * (sn1 + sn2);
				snneg[n - 1] = 0.5 * (sn1 - sn2);
			}
			R = 0;
			for (int i = (int) (0.005 * time.size()); i < imax; i++) {
				linos = (linos1 + linosMethod(snneg, snpos, lambda, my, i,
						newDeff)) * maxCurrent;
				R += Math.pow((current.get((int) i) - linos), 2);
			}
			rList.add((double) R);
			list.add(my);
		}

		// Retrieving index of best match, saving residual to list and returning
		// my
		int minIndex = rList.indexOf(Collections.min(rList));
		endList.add(rList.get(minIndex));
		return list.get(minIndex);
	}

	// Method utilizing Linos curve to apply a low pass filter
	public void linosLow() {

		// declaring temporary holder variables
		double lino;
		double trans;
		double average;
		boolean flag = true;

		// Looking for abnormal values and replacing them with the lino value
		while (flag) {
			flag = false;
			for (int i = (int) (time.size() * 0.005); i < time.size() - 2; i++) {
				trans = (double) (transient1.getY(i));
				lino = (double) (linosFit.getY(i));
				System.out.println(trans - lino);
				if (trans - lino > 0.02962) {
					System.out.println("Hei");
					flag = true;
					transient1.remove(i);
					average = ((double) transient1.getY(i - 1) + (double) transient1
							.getY(i + 1)) / 2;
					transient1.add(transient1.getX(i), average);
					current.set(i, (float) (average / 1000000));
				}

			}
		}
	}

	// Method utilizing Linos curve to apply a high pass filter
	public void linosHigh() {

		// declaring temporary holder variables
		double lino;
		double trans;
		double average;
		boolean flag = true;

		// Looking for abnormal values and replacing them with the lino value
		while (flag) {
			flag = false;
			for (int i = (int) (time.size() * 0.005); i < time.size() - 2; i++) {
				trans = (double) (transient1.getY(i));
				lino = (double) (linosFit.getY(i));
				if (lino - trans > 0.02962) {
					flag = true;
					transient1.remove(i);
					average = ((double) transient1.getY(i - 1) + (double) transient1
							.getY(i + 1)) / 2;
					transient1.add(transient1.getX(i), average);
					current.set(i, (float) (average));
				}

			}
		}
	}

	public void ficksFit() {

		// Declaring and initializing the scaled max current
		double maxCurrent = (double) Collections.max(current) * 1000000;

		// Declaring max current value from experiment
		int imax = current.indexOf(Collections.max(current));

		// Remove potential existing data
		ficksFit.clear();
		dataset.removeSeries(ficksFit);

		// Declare and initialize variables
		double temp;
		double lastR = 200;
		double newDeff = deff;
		boolean flag = false;
		double R;
		double ficks;

		// Finding the best match for the diffusion coefficient, using boundary
		// and
		// initial conditions, and the least Squares method.
		while (!flag) {
			newDeff -= 9E-9;
			R = 0;
			for (double i = (int) (0.01 * time.size()); i < imax; i++) {
				R += Math
						.pow((current.get((int) i) - (1 + 2
								* (Math.exp(-36 * Math.PI * Math.PI
										* ((3 * i * newDeff) / (L * L)))
										- Math.exp(-25 * Math.PI * Math.PI
												* ((3 * i * newDeff) / (L * L)))
										+ Math.exp(-16 * Math.PI * Math.PI
												* ((3 * i * newDeff) / (L * L)))
										- Math.exp(-9 * Math.PI * Math.PI
												* ((3 * i * newDeff) / (L * L)))
										+ Math.exp(-4 * Math.PI * Math.PI
												* ((3 * i * newDeff) / (L * L))) - Math
											.exp(-Math.PI
													* Math.PI
													* ((3 * i * newDeff) / (L * L))))
								* maxCurrent)), 2);
			}
			if (lastR - R < 0) {
				flag = true;
			}
			lastR = R;
		}

		// Computing and adding value for each experimental element
		ficksDeff = (float) newDeff;
		for (double i = 1; i < time.size(); i++) {
			ficks = (1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((3 * i * newDeff) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L))) - Math
						.exp(-Math.PI * Math.PI * ((3 * i * newDeff) / (L * L)))))
					* maxCurrent;
			temp = (time.get((int) i));
			ficksFit.add(temp, ficks);
		}

		// Add series to dataset
		dataset.addSeries(ficksFit);
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;
		float temp;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (transient1.getY(i));
			temp = (float) db;
			if (temp - lastValue > 0.02962) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;
		float temp;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (transient1.getY(i));
			temp = (float) db;
			if (temp - lastValue < 0.02962) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {

		// Removing existing data and inputting new
		transient1.clear();
		transient2.clear();
		transient3.clear();
		ficksElapsed.clear();
		ficksFit.clear();
		ficksCurve.clear();
		linosFit.clear();
		double scaleCurrent;
		double temp;

		// Adding the main transient data
		for (int i = 0; i < time.size(); i++) {
			scaleCurrent = (double) current.get(i) * 1000000;
			temp = time.get(i);
			transient1.add(temp, scaleCurrent);
		}
	}

	// Implementing writer method for transient 2
	public void transient2(BufferedReader br) throws IOException {

		// Removing potential data
		transient2.clear();
		dataset.removeSeries(transient2);

		// Declaring local variables
		String line;
		String[] values;

		// Reading file and adding to series
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			transient2.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}

		// Adding series to dataset
		dataset.addSeries(transient2);
		br.close();
	}

	// Implementing writer method for transient 2
	public void transient3(BufferedReader br) throws IOException {

		// Removing potential data
		transient3.clear();
		dataset.removeSeries(transient3);

		// Declaring local variables
		String line;
		String[] values;

		// Reading file and adding to series
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			transient3.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}

		// Adding series to dataset
		dataset.addSeries(transient3);
		br.close();
	}

	// Implementing a method to save series data locally, to be used later
	public void save(FileWriter fw) throws IOException {

		// Writing every point to a line seperated by a tab in a text document
		for (int i = 0; i < time.size(); i++) {
			fw.write(transient1.getX(i) + "\t" + transient1.getY(i)
					+ System.getProperty("line.separator"));
		}
	}

	// Refreshing data in the Resultsview
	public void refreshResultsArea(JTextArea resultsArea) {

		// remove initial data
		resultsArea.setText(null);

		// Append data to the result area
		resultsArea.append("CHART RESULTS" + "\n" + "Ficks Deff" + "\t"
				+ ficksDeff + "\n" + "Linos Deff" + "\t" + linosDeff + "\n"
				+ "Linos Kappa" + "\t" + linosKappa + "\n" + "Linos Lambda"
				+ "\t" + linosLambda + "\n" + "Linos My" + "\t" + linosMy
				+ "\n" + "Linos Residual" + "\t" + linosResidual);
	}

}
